package digital.laboratory.platform.identifybook.controller;


import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBook;
import digital.laboratory.platform.identifybook.entity.IdentifyBookRelevance;
import digital.laboratory.platform.identifybook.service.IIdentifyBookService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 鉴定文书信息表 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-22
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定文书详细信息相关接口")
@RequestMapping("/book/identifyBook")
public class IdentifyBookController {

    private final OssFile ossFile;

    private final IIdentifyBookService identifyBookService;

    @ApiOperation(value = "根据根据文书ID查看文书", notes = "根据根据文书ID查看文书")
    @ApiImplicitParam(name = "bookId", value = "文书ID", required = true)
    @GetMapping("/view/{id}")
    public R findBook(@PathVariable(value = "bookId") String bookId) {
        try {

            IdentifyBook identifyBook = identifyBookService.getById(bookId);
            String uuid = IdWorker.get32UUID();
            return R.ok(uuid + "#" + identifyBook.getPath(), "获取文件成功");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.failed("获取鉴定文书失败");
    }

    @ApiOperation(value = "根据根据文书ID删除文书", notes = "根据根据文书ID删除文书")
    @ApiImplicitParam(name = "bookId", value = "文书ID", required = true)
    @DeleteMapping
    public R deleteById(@PathVariable(value = "bookId") String bookId) {
        try {
            IdentifyBook identifyBook = identifyBookService.getById(bookId);
            ossFile.fileDelete(identifyBook.getFileName());
            boolean b = identifyBookService.removeById(bookId);
            if (b) {
                return R.ok(b, "删除成功！！！");
            }
            return R.failed(b, "删除失败！！！");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.failed("获取鉴定文书失败");
    }

    @PostMapping("/uploadBookAttachment")
    @ApiOperation(value = "上传文书的附件，主要包含 鉴定事项确认书(1)，委托书(2)，检材照片(3)，送检人证件照(4)，原鉴定情况(5)", notes = "上传文书的附件，主要包含 鉴定事项确认书(1)，委托书(2)，检材照片(3)，送检人证件照(4)，原鉴定情况(5)")
    public R uploadBookAttachment(String attachType, String businessId, List<MultipartFile> file) {
        Assert.notBlank(attachType, "附件类型参数不能为空");
        Assert.notBlank(businessId, "业务ID参数不能为空");
        return R.ok(identifyBookService.uploadIdentifyBookAttachment(attachType, file, businessId));
    }

    //获取手动上传的文书附件
    @GetMapping("/getUploadBookAttachment")
    @ApiOperation(value = "获取手动上传的文书附件，主要包含 鉴定事项确认书(1)，委托书(2)，检材照片(3)，送检人证件照(4)，原鉴定情况(5)", notes = "上传文书的附件，主要包含 鉴定事项确认书(1)，委托书(2)，检材照片(3)，送检人证件照(4)，原鉴定情况(5)")
    public R getAttachment(String businessId, String attachType) {
        return R.ok(identifyBookService.getAttachment(businessId, attachType));
    }

    @GetMapping("/mergeWord")
    @ApiOperation(value = "合并所有", notes = "合并word")
    public R mergeWord(String entrustId) throws Exception {
        return R.ok(identifyBookService.mergeWord(entrustId));
    }
}
