package digital.laboratory.platform.identifybook.controller;


import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBook;
import digital.laboratory.platform.identifybook.entity.IdentifyBookRelevance;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookRelevanceService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookService;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 鉴定文书关联中间表 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-22
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定文书-相关接口")
@RequestMapping("/book/identifyBookRelevance")
public class IdentifyBookRelevanceController {

    private final OssFile ossFile;

    private final IIdentifyBookRelevanceService identifyBookRelevanceService;
    private final IIdentifyBookService identifyBookService;
    private final IEntrustmentService entrustmentService;
    private final RemoteUserService remoteUserService;

    @ApiOperation(value = "根据委托ID获取鉴定文书", notes = "根据委托ID获取鉴定文书")
    @ApiImplicitParam(name = "entrustId", value = "委托ID" , required = true)
    @GetMapping("/getBook/{entrustId}")
    public R getBook(@PathVariable(value = "entrustId") String entrustId, HttpServletResponse response){
        try {
            Entrustment entrustment = entrustmentService.getById(entrustId);
            List<String> entrustIds = new ArrayList<>();
            List<Entrustment> entrustmentList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                    .select(Entrustment::getId)
                    //.eq(Entrustment::getIdentificationId, entrustment.getIdentificationId())
                    .eq(Entrustment::getId, entrustment.getId())
            );
            if(CollectionUtils.isNotEmpty(entrustmentList)){
                entrustIds.addAll(entrustmentList.stream().map(m -> m.getId()).collect(Collectors.toList()));
            }
            IdentifyBookRelevance one = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .in(IdentifyBookRelevance::getRelevanceBusinessId, entrustIds));
            if (ObjectUtils.isNotEmpty(one)) {
                IdentifyBook identifyBook = identifyBookService.getMaxVersionIdentifyBook(one.getId());
                //ossFile.fileGet(identifyBook.getPath(), response.getOutputStream());
                String uuid= IdWorker.get32UUID();
                return R.ok( uuid+"#"+identifyBook.getPath(), "获取文件成功");
                //response.setContentType(new MimetypesFileTypeMap().getContentType(identifyBook.getFileName()));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.failed("获取鉴定文书失败");
    }

    @ApiOperation(value = "根据委托ID获取鉴定文书制作或修订历史列表", notes = "根据委托ID获取鉴定文书制作或修订历史列表")
    @ApiImplicitParam(name = "entrustId", value = "委托ID" , required = true)
    @GetMapping("/findBookHistory/{entrustId}")
    public R findBookHistory(@PathVariable(value = "entrustId") String entrustId){
        try {
            Entrustment entrustment = entrustmentService.getById(entrustId);
            List<String> entrustIds = new ArrayList<>();
            List<Entrustment> entrustmentList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                    .select(Entrustment::getId)
                    //.eq(Entrustment::getIdentificationId, entrustment.getIdentificationId())
                    .eq(Entrustment::getId,entrustId)
            );
            if(CollectionUtils.isNotEmpty(entrustmentList)){
                entrustIds.addAll(entrustmentList.stream().map(m -> m.getId()).collect(Collectors.toList()));
            }
            IdentifyBookRelevance one = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .in(IdentifyBookRelevance::getRelevanceBusinessId, entrustIds));
            if (ObjectUtils.isNotEmpty(one)) {
//                IdentifyBook identifyBook = identifyBookService.getMaxVersionIdentifyBook(one.getId());
                List<IdentifyBook> identifyBookList = identifyBookService.list(new LambdaQueryWrapper<IdentifyBook>()
                        .eq(IdentifyBook::getIdentifyBookRelevanceId, one.getId())
                        .orderByDesc(IdentifyBook::getCreateTime));
                for (IdentifyBook identifyBook: identifyBookList){
                    if(StringUtils.isNotEmpty(identifyBook.getCreateBy())){
                        identifyBook.setOperateUserName(remoteUserService.innerGetById(identifyBook.getCreateBy()).getData().getName());
                    }


                }
                return R.ok(identifyBookList);
            }
            return R.ok(null, "该委托还未制作文书！！！");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.failed("获取鉴定文书失败");
    }

}
