package digital.laboratory.platform.identifybook.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.identifybook.emums.BusinessStatus;
import digital.laboratory.platform.identifybook.entity.IdentifyBookRelevance;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookRelevanceService;
import digital.laboratory.platform.identifybook.service.ITaskNotificationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 公共数据统计 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-08-31
 */
@RestController
@RequiredArgsConstructor
@Api( tags = "公共数据统计-相关接口管理")
@RequestMapping("/book/commDataStatistics")
public class commDataStatisticsController {

    private final ITaskNotificationService taskNotificationService;

    private final IEntrustmentService entrustmentService;

    private final IIdentifyBookRelevanceService identifyBookRelevanceService;

    @ApiOperation(value = "综合统计", notes = "综合统计")
    @GetMapping("/comprehensiveStatistics")
    public R comprehensiveStatistics(HttpServletRequest theHttpServletRequest){
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        Map<String, Object> mapTotal= new HashMap<>();
        // 鉴定文书列表任务统计
        long countWSLBSum = entrustmentService.countWSLBSum(Stream.of(-1, -2, 3, -4, 5, 7).collect(Collectors.toList()));
        // 审核审批签发任务统计
        long countRWSum = taskNotificationService.countRWSum(dlpUser);
        // 鉴定文书签发领取任务统计
        long countQFLQSum = identifyBookRelevanceService.count(new LambdaQueryWrapper<IdentifyBookRelevance>()
                .eq(IdentifyBookRelevance::getBookStatus, BusinessStatus.JD_WS_SP_QF_TG.getCode()));

        mapTotal.put("countWSLBSum", countWSLBSum);
        mapTotal.put("countRWSum", countRWSum);
        mapTotal.put("countQFLQSum", countQFLQSum);
        return R.ok(mapTotal, "countWSLBSum：鉴定文书列表统计结果，countRWSum：审核审批任务统计结果，countQFLQSum：签发领取任务结果");

    }

    @ApiOperation(value = "审核审批任务统计", notes = "审核审批任务统计")
    @GetMapping("/reviewAndApprovalTaskStatistics")
    public R reviewAndApprovalTaskStatistics(HttpServletRequest theHttpServletRequest){
        // 获取当前登录用户信息
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        return R.ok(taskNotificationService.countRWSum(dlpUser));

    }

    @ApiOperation(value = "鉴定文书任务列表/待制作/退回修改任务统计", notes = "鉴定文书任务列表/待制作/退回修改任务统计")
    @GetMapping("/identifyBookDZZORTHXGTaskStatistics")
    public R identifyBookDZZORTHXGTaskStatistics(){
        Map<String, Object> mapTotal= new HashMap<>();
        // 鉴定文书任务列表/待制作统计
        long countWSLBDZZSum = entrustmentService.countWSLBSum(Stream.of(-1).collect(Collectors.toList()));
        // 鉴定文书任务列表/退回修改/补充
        long countWSLBTHXGSum = entrustmentService.countWSLBSum(Stream.of(3, 5).collect(Collectors.toList()));
        // 鉴定文书任务列表/重新制作
        long countWSLBCXZZSum = entrustmentService.countWSLBSum(Stream.of(-2, -4, 7).collect(Collectors.toList()));
        mapTotal.put("countWSLBDZZSum", countWSLBDZZSum);
        mapTotal.put("countWSLBTHXGSum", countWSLBTHXGSum);
        mapTotal.put("countWSLBCXZZSum", countWSLBCXZZSum);

        return R.ok(mapTotal, "countWSLBDZZSum：鉴定文书待制作任务统计结果、countWSLBTHXGSum：鉴定文书退回修改任务统计结果、countWSLBCXZZSum：鉴定文书任务列表/重新制作");

    }

    @ApiOperation(value = "鉴定文书签发待领取任务统计", notes = "鉴定文书签发待领取任务统计")
    @GetMapping("/identifyBookQFDLQTaskStatistics")
    public R identifyBookQFDLQTaskStatistics(){
        return R.ok(identifyBookRelevanceService.count(
                new LambdaQueryWrapper<IdentifyBookRelevance>()
                        .eq(IdentifyBookRelevance::getBookStatus, BusinessStatus.JD_WS_SP_QF_TG.getCode())));

    }


}
