package digital.laboratory.platform.identifybook.emums;

import lombok.Getter;

import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ChenJiangBao
 * @version 1.0
 * @description: 附件类型集合（1：检验记录（含原始记录，检验图表）、2：鉴定事项确认书、3：鉴定委托书或鉴定聘请书、4：检材照片、5：送检人证件复印件、6：原鉴定文书）
 * @date 2025/4/2 10:12
 */
@Getter
public enum AnnexTypeEnum {
    // 附件类型集合（1：检验记录（含原始记录，检验图表）、2：鉴定事项确认书、3：鉴定委托书或鉴定聘请书、4：检材照片、5：送检人证件复印件、6：原鉴定文书）
    INSPECTION_RECORD("1", "检验记录（含原始记录，检验图表）"),
    IDENTIFICATION_CONFIRMATION("2", "鉴定事项确认书"),
    ENTRUST_LETTER("3", "鉴定委托书或鉴定聘请书"),
    MATERIAL_PHOTO("4", "检材照片"),
    EXAMINER_ID_COPY("5", "送检人证件复印件"),
    ORIGINAL_DOCUMENT("6", "原鉴定文书"),
    ;

    private final String code;
    private final String desc;

    AnnexTypeEnum(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    // 使用 Map 提高查找效率
    private static final Map<String, AnnexTypeEnum> CODE_MAP =
            Arrays.stream(AnnexTypeEnum.values())
                    .collect(Collectors.toMap(AnnexTypeEnum::getCode, e -> e));

    // 根据 code 获取枚举
    public static AnnexTypeEnum fromCode(String code) {
        return CODE_MAP.get(code);
    }
}
