package digital.laboratory.platform.identifybook.emums;

import lombok.Getter;

import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ChenJiangBao
 * @version 1.0
 * @description: 报告领取方式: 0=自取, 1=代领, 2=邮寄
 * @date 2025/4/2 10:12
 */
@Getter
public enum ReceiveWayEnum {
    // 报告领取方式: 0=自取, 1=代领, 2=邮寄
    SELF_PICKUP("0", "自取"),
    PROXY_PICKUP("1", "代领"),
    MAIL("2", "邮寄");

    private final String code;
    private final String desc;

    ReceiveWayEnum(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    // 使用 Map 提高查找效率
    private static final Map<String, ReceiveWayEnum> CODE_MAP =
            Arrays.stream(ReceiveWayEnum.values())
                    .collect(Collectors.toMap(ReceiveWayEnum::getCode, e -> e));

    // 根据 code 获取枚举
    public static ReceiveWayEnum fromCode(String code) {
        return CODE_MAP.get(code);
    }
}
