package digital.laboratory.platform.identifybook.entity;

import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.IdType;

import java.time.LocalDateTime;
import java.util.Date;
import com.baomidou.mybatisplus.annotation.TableId;
import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import digital.laboratory.platform.common.mybatis.base.BaseEntity;
import digital.laboratory.platform.sys.entity.DrugLite;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * <p>
 * 检验用的样本
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-17
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName(value = "b_sample", autoResultMap = true)
@ApiModel(value="Sample对象", description="检验用的样本")
public class Sample extends BaseEntity {

    /**
     * 样本id
     */
    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    @ApiModelProperty(value = "样本id")
    private String id;

    /**
     * 检材id, 对应委托的检材id或毛发任务的检材id或污水任务的检材id
     */
    @ApiModelProperty(value = "检材id, 对应委托的检材id或毛发任务的检材id或污水任务的检材id")
    private String identificationMaterialId;

    /**
     * 样本编号
     */
    @ApiModelProperty(value = "样本编号")
    private String sampleNo;

    /**
     * 样本来源: entrustment/hairJob/sewageJob/....
     */
    @ApiModelProperty(value = "样本来源: entrustment/hairJob/sewageJob/....")
    private String source;

    /**
     * 委托id
     */
    @ApiModelProperty(value="委托id")
    private String entrustmentId;

    /**
     * 污水任务id
     */
    @ApiModelProperty(value="污水任务id")
    private String sewageJobId;

    /**
     * 毛发任务id
     */
    @ApiModelProperty(value="毛发任务id")
    private String hairJobId;

    /**
     * 定性分析
     */
    @ApiModelProperty(value="定性分析")
    private Boolean qualitativeAnalysis;

    /**
     * 定量分析
     */
    @ApiModelProperty(value="定量分析")
    private Boolean quantitativeAnalysis;

    /**
     * 候选毒品列表(drug 对象的 json array)
     */
    @ApiModelProperty(value="候选毒品列表(drug 对象的 json array)")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private List<DrugLite> candidateDrugs;

    /**
     * 检材情况之承载物名称, 例如 棉签 2 棵, 粉末 少许
     */
    @ApiModelProperty(value="检材情况之承载物名称, 例如 棉签 2 棵, 粉末 少许")
    private String fundName;

    /**
     * 检材情况之承载物数量, 如 3 颗 或 1包
     */
    @ApiModelProperty(value="检材情况之承载物数量, 如 3 颗 或 1包")
    private Integer fundQuantity;

    /**
     * 检材情况之承载物单位, 例如 5 颗, 3包
     */
    @ApiModelProperty(value="检材情况之承载物单位, 例如 5 颗, 3包")
    private String fundUnit;

    /**
     * 样品数量, 例如 2.3 克 或 3.1 毫升
     */
    @ApiModelProperty(value="样品数量, 例如 2.3 克 或 3.1 毫升")
    private BigDecimal quantity;

    /**
     * 计量单位, 例如 3.8 克 或 4.5毫升
     */
    @ApiModelProperty(value="计量单位, 例如 3.8 克 或 4.5毫升")
    private String unit;

    /**
     * 附加属性, 如:体积3毫升,密码123,型号abc
     */
    @ApiModelProperty(value="附加属性, 如:体积3毫升,密码123,型号abc")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private List<IMAdditionalProperty> additionalProperties;

    /**
     * 提取方法
     */
    @ApiModelProperty(value="提取方法")
    private String drawWay;

    /**
     * 盒子编号
     */
    @ApiModelProperty(value="盒子编号")
    private String boxId;

    /**
     * A样还是B样
     */
    @ApiModelProperty(value="A样还是B样")
    private String sampleType;

    /**
     * 存储方法:常规,冷藏,特殊
     */
    @ApiModelProperty(value="存储方法:常规,冷藏,特殊")
    private String storageMethod;

    /**
     * 受理时间, 相当于检验开始时间, 用于比对花了多少时间在检验上
     */
    @ApiModelProperty(value="受理时间, 相当于检验开始时间, 用于比对花了多少时间在检验上")
    private LocalDateTime acceptTime;

    /**
     * 状态: 0=已受理,等分配, 1=已分配到检验人员, 2=检验完成
     */
//    @ApiModelProperty(value="状态: 0=已受理,等分配, 1=已分配到检验人员, 2=检验完成")
//    private Integer status;
    @ApiModelProperty(value = "状态: 0=已受理,等分配, 1=已分配到检验人员, 2=检验完成")
    private IMTSampleStatus status;

    /**
     * 当前持有者
     */
    @ApiModelProperty(value="当前持有者")
    private String holder;

    /**
     * 送检单位id
     */
    @ApiModelProperty(value="送检单位id")
    private String orgId;

    /**
     * 样本名称
     */
    @ApiModelProperty(value="样本名称")
    private String name;

    @ApiModelProperty(value = "检材受理编号")
    private String acceptNo;


}
