package digital.laboratory.platform.identifybook.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.identifybook.dto.EntrustCaseDTO;
import digital.laboratory.platform.identifybook.dto.EntrustmentDTO;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBookRelevance;
import digital.laboratory.platform.identifybook.mapper.EntrustmentMapper;
import digital.laboratory.platform.identifybook.req.EntrustmentREQ;
import digital.laboratory.platform.identifybook.service.IEntrustmentBundleService;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IExperimentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookRelevanceService;
import digital.laboratory.platform.inspetion.api.entity.EntrustInfo;
import digital.laboratory.platform.inspetion.api.feign.RemoteTestToIdentifyService;
import digital.laboratory.platform.inspetion.api.vo.TestRecordVo;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 委托 服务实现类
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-17
 */
@Service
public class EntrustmentServiceImpl extends ServiceImpl<EntrustmentMapper, Entrustment> implements IEntrustmentService {

    @Autowired
    private IExperimentService experimentService;
    @Autowired
    private IEntrustmentBundleService entrustmentBundleService;
    @Resource
    private RemoteUserService remoteUserService;
    @Resource
    private RemoteTestToIdentifyService remoteTestToIdentifyService;
    @Resource
    private IIdentifyBookRelevanceService identifyBookRelevanceService;

    private final SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH时mm分ss秒");

    @Override
    public void synchronizationTestSysData(List<String> synedIdList) {
        R testFinishBusinessData = remoteTestToIdentifyService.getTestFinishBusinessData(synedIdList);
        String retData = JSON.toJSONString(testFinishBusinessData.getData());

        List<EntrustInfo> entrustInfoList = JSONArray.parseArray(retData, EntrustInfo.class);
        List<Entrustment> bookEntrustList = new ArrayList<>();
        for (EntrustInfo entrustInfo : entrustInfoList) {
            Entrustment eg = new Entrustment();
            eg.setId(entrustInfo.getId());
            eg.setSynEntrustId(entrustInfo.getSynEntrustId());
            eg.setDataSources(entrustInfo.getSource());// 0 系统推送，1 检验系统手工录入的
            eg.setBusinessType(Integer.parseInt(entrustInfo.getBusinessType()));
            eg.setCaseName(entrustInfo.getCaseName());
            eg.setCaseBrief(entrustInfo.getCaseBrief());
            eg.setAcceptNo(entrustInfo.getAcceptNo());
            eg.setAcceptTime(entrustInfo.getAcceptDate());
            eg.setEntrustmentNo(entrustInfo.getEntrustNo());
            eg.setClientOrgName(entrustInfo.getEntrustDepartment());
            eg.setIdentificationOrgName(entrustInfo.getIdentityDepartment());
            eg.setEntrustRequirement(entrustInfo.getEntrustRequirement());
            //送检人信息
            eg.setDeliverer1Name(entrustInfo.getDeliver1Name());
            eg.setDeliverer1Phone(entrustInfo.getDeliver1Phone());
            eg.setDeliverer1Position(entrustInfo.getDeliver1Position());
            eg.setDeliverer1Cert(entrustInfo.getDeliver1Cert());
            eg.setDeliverer1Id(entrustInfo.getDeliver1No());

            eg.setDeliverer2Name(entrustInfo.getDeliver2Name());
            eg.setDeliverer2Phone(entrustInfo.getDeliver2Phone());
            eg.setDeliverer2Position(entrustInfo.getDeliver2Position());
            eg.setDeliverer2Cert(entrustInfo.getDeliver2Cert());
            eg.setDeliverer2Id(entrustInfo.getDeliver2No());
            // 检材数量
            eg.setMaterialNum(entrustInfo.getMaterialNum());
            // 设置文书领取方式
            eg.setReportReceiveMode(entrustInfo.getReportReceiveMode());
            eg.setPostAddress(entrustInfo.getPostAddress());

            //eg.setReportReceiveMode(entrustInfo);
            eg.setUpdateBy(entrustInfo.getUpdateBy());
            eg.setUpdateTime(entrustInfo.getUpdateTime());
            eg.setCreateBy(entrustInfo.getCreateBy());
            eg.setCreateTime(entrustInfo.getCreateTime());
            bookEntrustList.add(eg);
        }
        List<IdentifyBookRelevance> identifyBookRelevanceList = new ArrayList<>();
        for (Entrustment entrustment : bookEntrustList) {
            IdentifyBookRelevance identifyBookRelevance = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .eq(IdentifyBookRelevance::getRelevanceBusinessId, entrustment.getId()));
            if (ObjectUtils.isEmpty(identifyBookRelevance)) {
                String[] arrNo = entrustment.getAcceptNo().split("-");
                identifyBookRelevance = new IdentifyBookRelevance();
                identifyBookRelevance.setRelevanceBusinessId(entrustment.getId());
                identifyBookRelevance.setYear(arrNo[0]);
                identifyBookRelevance.setSerialNumber(arrNo[1]);
                identifyBookRelevance.setBookStatus(-1);
                identifyBookRelevanceList.add(identifyBookRelevance);
            }
        }
        if (bookEntrustList.size() > 0) {
            boolean entrustSuccess = this.saveOrUpdateBatch(bookEntrustList);//保存到文书系统的 entrust中
            if (!entrustSuccess) {
                throw new RuntimeException("保存委托信息失败！！！");
            }
        }
        if (identifyBookRelevanceList.size() > 0) {
            boolean ibrSuccess = identifyBookRelevanceService.saveBatch(identifyBookRelevanceList);
            if (!ibrSuccess) {
                throw new RuntimeException("保存文书关联信息失败！！！");
            }
        }
    }

    @Override
    public EntrustCaseDTO getEntrustCaseDTOById(String entrustId) {
        return baseMapper.getEntrustCaseDTOById(entrustId);
    }

    @Override
    public IPage<EntrustmentDTO> findEntrustPage(IPage<EntrustmentDTO> page, EntrustmentREQ req) {
        IPage<EntrustmentDTO> entrustPage = baseMapper.findEntrustPage(page, req);
//        for (EntrustmentDTO e: entrustPage.getRecords()) {
//            List<EntrustmentDTO> entrustmentDTOList = baseMapper.findEntrust(e.getId(), e.getIdentificationId());
//
//            List<String> stringIDSList = Stream.of(e.getId()).collect(Collectors.toList());
//
//            if(CollectionUtils.isNotEmpty(entrustmentDTOList)){
//                stringIDSList.addAll(
//                        entrustmentDTOList
//                                .stream()
//                                .map(m -> m.getId())
//                                .collect(Collectors.toList()));
//                e.setChildren(entrustmentDTOList);
//            }
//            // 根据委托ID获取实验与委托关联消息-取实验信息
//            List<EntrustmentBundle> entrustmentBundleList = entrustmentBundleService.list(new LambdaQueryWrapper<EntrustmentBundle>()
//                    .in(EntrustmentBundle::getId, stringIDSList));
//            List<ExperimentVO> experimentVOList = new ArrayList<>();
//            List<String> experimentAllIds = new ArrayList<>();
//            if(CollectionUtils.isNotEmpty(entrustmentBundleList)){
//                for (EntrustmentBundle entrustmentBundle: entrustmentBundleList){
//                    List<String> experimentIds = entrustmentBundle.getSelectedExperiment();
//                    experimentAllIds.addAll(experimentIds);
//                    for (String experimentId : experimentIds) {
//                        Experiment experiment = experimentService.getById(experimentId);
//                        ExperimentVO experimentVO = new ExperimentVO();
//                        BeanUtil.copyProperties(experiment, experimentVO);
//                        experimentVO.setOpUserName(remoteUserService.innerGetById(experimentVO.getOpUserId()).getData().getName());
//                        experimentVOList.add(experimentVO);
//                    }
//                }
//
//            }
//            List<Date> minStartDateList = experimentVOList.stream().map(m -> m.getCreateTime()).collect(Collectors.toList());
//            List<Date> maxEndDateList = experimentVOList.stream().map(m -> m.getFinishDate()).collect(Collectors.toList());
//            e.setAppraisalUserName(experimentVOList.get(0).getOpUserName());
//            e.setInspectionStartDate(sdf.format(Collections.min(minStartDateList)));
//            e.setFinishDate(sdf.format(Collections.max(maxEndDateList)));
//        }
        List<EntrustmentDTO> records = entrustPage.getRecords();
        if (CollUtil.isNotEmpty(records)) {
            List<String> businessIds = records.stream().map(EntrustmentDTO::getId).collect(Collectors.toList());
            R<Map<String, TestRecordVo>> r = remoteTestToIdentifyService.queryTestRecordInfoByBusinessId(businessIds);
            if (r.getCode() == CommonConstants.SUCCESS) {
                Map<String, TestRecordVo> dataMap = r.getData();
                DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日");
                for (EntrustmentDTO record : records) {
                    getTestDateAndAppraiser(record, dataMap, dateTimeFormatter);
                }
            } else {
                log.error("==================>>>>>>>>>>>>>>>>>> 获取实验信息失败！");
            }
        }
        entrustPage.setRecords(records);
        return entrustPage;
    }

    @Override
    public EntrustmentDTO getEntrustmentDTO(String entrustId) {

        EntrustmentDTO entrustmentDTO = baseMapper.getEntrustmentDTO(entrustId);
        R<Map<String, TestRecordVo>> r = remoteTestToIdentifyService.queryTestRecordInfoByBusinessId(Arrays.asList(entrustmentDTO.getId()));
        if (r.getCode() == CommonConstants.SUCCESS) {
            Map<String, TestRecordVo> dataMap = r.getData();
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日");
            getTestDateAndAppraiser(entrustmentDTO, dataMap, dateTimeFormatter);
        } else {
            log.error("==================>>>>>>>>>>>>>>>>>> 获取实验信息失败！");
        }
        return entrustmentDTO;
//
//        List<EntrustmentDTO> entrustmentDTOList = baseMapper.findEntrust(entrustmentDTO.getId(), entrustmentDTO.getIdentificationId());
//
//        List<String> stringIDSList = Stream.of(entrustmentDTO.getId()).collect(Collectors.toList());
//
//        if(CollectionUtils.isNotEmpty(entrustmentDTOList)){
//
//            stringIDSList.addAll(
//                    entrustmentDTOList
//                            .stream()
//                            .map(m -> m.getId())
//                            .collect(Collectors.toList()));
//        }
//        // 根据委托ID获取实验与委托关联消息-取实验信息
//        List<EntrustmentBundle> entrustmentBundleList = entrustmentBundleService.list(new LambdaQueryWrapper<EntrustmentBundle>()
//                .in(EntrustmentBundle::getId, stringIDSList));
//        List<ExperimentVO> experimentVOList = new ArrayList<>();
//        List<String> experimentAllIds = new ArrayList<>();
//        if(CollectionUtils.isNotEmpty(entrustmentBundleList)){
//            for (EntrustmentBundle entrustmentBundle: entrustmentBundleList){
//                List<String> experimentIds = entrustmentBundle.getSelectedExperiment();
//                experimentAllIds.addAll(experimentIds);
//                for (String experimentId : experimentIds) {
//                    Experiment experiment = experimentService.getById(experimentId);
//                    ExperimentVO experimentVO = new ExperimentVO();
//                    BeanUtil.copyProperties(experiment, experimentVO);
//                    experimentVO.setOpUserName(remoteUserService.innerGetById(experimentVO.getOpUserId()).getData().getName());
//                    experimentVOList.add(experimentVO);
//                }
//            }
//
//        }
//        List<Date> minStartDateList = experimentVOList.stream().map(m -> m.getCreateTime()).collect(Collectors.toList());
//        List<Date> maxEndDateList = experimentVOList.stream().map(m -> m.getFinishDate()).collect(Collectors.toList());
//        entrustmentDTO.setAppraisalUserName(experimentVOList.get(0).getOpUserName());
//        entrustmentDTO.setInspectionStartDate(sdf.format(Collections.min(minStartDateList)));
//        entrustmentDTO.setFinishDate(sdf.format(Collections.max(maxEndDateList)));


    }

    /**
     * 提取出检验时间和鉴定人名称
     *
     * @param entrustmentDTO
     * @param dataMap
     * @param dateTimeFormatter
     */
    private void getTestDateAndAppraiser(EntrustmentDTO entrustmentDTO, Map<String, TestRecordVo> dataMap, DateTimeFormatter dateTimeFormatter) {
        TestRecordVo testRecordVo = dataMap.get(entrustmentDTO.getId());
        if (testRecordVo != null && testRecordVo.getStatus() == 5) {
            entrustmentDTO.setAppraisalUserName(testRecordVo.getTestUserName());
            entrustmentDTO.setInspectionStartDate(testRecordVo.getTestStartDate().format(dateTimeFormatter));
            entrustmentDTO.setFinishDate(testRecordVo.getTestEndDate().format(dateTimeFormatter));
        } else {
            log.warn("未从检验鉴定中获取到文书对应的实验信息。");
        }
    }

    @Override
    public long countWSLBSum(List<Integer> bookStatusList) {
        return baseMapper.countWSLBSum(bookStatusList);
    }
}
